@@ -25,7 +25,7 @@ import java.util.concurrent.*;
 import android.util.Log;
 import android.os.Handler;
 
-import org.altusmetrum.AltosLib.*;
+import org.altusmetrum.altoslib_1.*;
 
 
 public class TelemetryReader extends Thread {
@@ -68,12 +68,12 @@ public class TelemetryReader extends Thread {
 					if (record == null)
 						break;
 					state = new AltosState(record, state);
-
 					handler.obtainMessage(TelemetryService.MSG_TELEMETRY, state).sendToTarget();
 				} catch (ParseException pp) {
 					Log.e(TAG, String.format("Parse error: %d \"%s\"", pp.getErrorOffset(), pp.getMessage()));
 				} catch (AltosCRCException ce) {
 					++crc_errors;
+					handler.obtainMessage(TelemetryService.MSG_CRC_ERROR, new Integer(crc_errors)).sendToTarget();
 				}
 			}
 		} catch (InterruptedException ee) {
