@@ -18,7 +18,7 @@
 /*
  * Sensor data conversion functions
  */
-package org.altusmetrum.AltosLib;
+package org.altusmetrum.altoslib_1;
 
 public class AltosConvert {
 	/*
@@ -258,6 +258,10 @@ public class AltosConvert {
 		return meters / 9.80665;
 	}
 
+	public static double c_to_f(double c) {
+		return c * 9/5 + 32;
+	}
+
 	public static boolean imperial_units = false;
 
 	public static AltosDistance distance = new AltosDistance();
@@ -268,6 +272,8 @@ public class AltosConvert {
 
 	public static AltosAccel accel = new AltosAccel();
 
+	public static AltosTemperature temperature = new AltosTemperature();
+
 	public static String show_gs(String format, double a) {
 		a = meters_to_g(a);
 		format = format.concat(" g");
