@@ -15,15 +15,13 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.AltosLib;
+package org.altusmetrum.altoslib_1;
 
 import java.io.*;
 import java.util.*;
-import java.util.prefs.*;
-import javax.swing.filechooser.FileSystemView;
 
 public class AltosPreferences {
-	public static Preferences preferences;
+	public static AltosPreferencesBackend backend = null;
 
 	/* logdir preference name */
 	public final static String logdirPreference = "LOGDIR";
@@ -64,6 +62,9 @@ public class AltosPreferences {
 	/* Log directory */
 	public static File logdir;
 
+	/* Last log directory - use this next time we open or save something */
+	public static File last_logdir;
+
 	/* Map directory -- hangs of logdir */
 	public static File mapdir;
 
@@ -100,13 +101,10 @@ public class AltosPreferences {
 	public static AltosFrequency[] load_common_frequencies() {
 		AltosFrequency[] frequencies = null;
 		boolean	existing = false;
-		try {
-			existing = preferences.nodeExists(common_frequencies_node_name);
-		} catch (BackingStoreException be) {
-			existing = false;
-		}
+		existing = backend.nodeExists(common_frequencies_node_name);
+
 		if (existing) {
-			Preferences	node = preferences.node(common_frequencies_node_name);
+			AltosPreferencesBackend	node = backend.node(common_frequencies_node_name);
 			int		count = node.getInt(frequency_count, 0);
 
 			frequencies = new AltosFrequency[count];
@@ -115,7 +113,7 @@ public class AltosPreferences {
 				String	description;
 
 				frequency = node.getDouble(String.format(frequency_format, i), 0.0);
-				description = node.get(String.format(description_format, i), null);
+				description = node.getString(String.format(description_format, i), null);
 				frequencies[i] = new AltosFrequency(frequency, description);
 			}
 		} else {
@@ -129,28 +127,27 @@ public class AltosPreferences {
 	}
 
 	public static void save_common_frequencies(AltosFrequency[] frequencies) {
-		Preferences	node = preferences.node(common_frequencies_node_name);
+		AltosPreferencesBackend	node = backend.node(common_frequencies_node_name);
 
 		node.putInt(frequency_count, frequencies.length);
 		for (int i = 0; i < frequencies.length; i++) {
 			node.putDouble(String.format(frequency_format, i), frequencies[i].frequency);
-			node.put(String.format(description_format, i), frequencies[i].description);
+			node.putString(String.format(description_format, i), frequencies[i].description);
 		}
 	}
 	public static int launcher_serial;
 
 	public static int launcher_channel;
 
-	public static void init() {
-		preferences = Preferences.userRoot().node("/org/altusmetrum/altosui");
+	public static void init(AltosPreferencesBackend in_backend) {
+		backend = in_backend;
 
 		/* Initialize logdir from preferences */
-		String logdir_string = preferences.get(logdirPreference, null);
+		String logdir_string = backend.getString(logdirPreference, null);
 		if (logdir_string != null)
 			logdir = new File(logdir_string);
 		else {
-			/* Use the file system view default directory */
-			logdir = new File(FileSystemView.getFileSystemView().getDefaultDirectory(), logdirName);
+			logdir = new File(backend.homeDirectory(), logdirName);
 			if (!logdir.exists())
 				logdir.mkdirs();
 		}
@@ -162,17 +159,17 @@ public class AltosPreferences {
 
 		telemetries = new Hashtable<Integer,Integer>();
 
-		voice = preferences.getBoolean(voicePreference, true);
+		voice = backend.getBoolean(voicePreference, true);
 
-		callsign = preferences.get(callsignPreference,"N0CALL");
+		callsign = backend.getString(callsignPreference,"N0CALL");
 
-		scanning_telemetry = preferences.getInt(scanningTelemetryPreference,(1 << AltosLib.ao_telemetry_standard));
+		scanning_telemetry = backend.getInt(scanningTelemetryPreference,(1 << AltosLib.ao_telemetry_standard));
 
-		launcher_serial = preferences.getInt(launcherSerialPreference, 0);
+		launcher_serial = backend.getInt(launcherSerialPreference, 0);
 
-		launcher_channel = preferences.getInt(launcherChannelPreference, 0);
+		launcher_channel = backend.getInt(launcherChannelPreference, 0);
 
-		String firmwaredir_string = preferences.get(firmwaredirPreference, null);
+		String firmwaredir_string = backend.getString(firmwaredirPreference, null);
 		if (firmwaredir_string != null)
 			firmwaredir = new File(firmwaredir_string);
 		else
@@ -180,65 +177,69 @@ public class AltosPreferences {
 
 		common_frequencies = load_common_frequencies();
 
-		AltosConvert.imperial_units = preferences.getBoolean(unitsPreference, false);
+		AltosConvert.imperial_units = backend.getBoolean(unitsPreference, false);
 	}
 
-	static { init(); }
-
 	public static void flush_preferences() {
-		try {
-			preferences.flush();
-		} catch (BackingStoreException ee) {
-/*
-			if (component != null)
-				JOptionPane.showMessageDialog(component,
-							      preferences.absolutePath(),
-							      "Cannot save prefernces",
-							      JOptionPane.ERROR_MESSAGE);
-			else
-*/
-				System.err.printf("Cannot save preferences\n");
-		}
+		backend.flush();
 	}
 
 	public static void set_logdir(File new_logdir) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			logdir = new_logdir;
 			mapdir = new File(logdir, "maps");
 			if (!mapdir.exists())
 				mapdir.mkdirs();
-			preferences.put(logdirPreference, logdir.getPath());
+			backend.putString(logdirPreference, logdir.getPath());
 			flush_preferences();
 		}
 	}
 
 	public static File logdir() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return logdir;
 		}
 	}
 
+	public static File last_logdir() {
+		synchronized (backend) {
+			if (last_logdir == null)
+				last_logdir = logdir;
+			return last_logdir;
+		}
+	}
+
+	public static void set_last_logdir(File file) {
+		synchronized(backend) {
+			if (file != null && !file.isDirectory())
+				file = file.getParentFile();
+			if (file == null)
+				file = new File(".");
+			last_logdir = file;
+		}
+	}
+
 	public static File mapdir() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return mapdir;
 		}
 	}
 
 	public static void set_frequency(int serial, double new_frequency) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			frequencies.put(serial, new_frequency);
-			preferences.putDouble(String.format(frequencyPreferenceFormat, serial), new_frequency);
+			backend.putDouble(String.format(frequencyPreferenceFormat, serial), new_frequency);
 			flush_preferences();
 		}
 	}
 
 	public static double frequency(int serial) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			if (frequencies.containsKey(serial))
 				return frequencies.get(serial);
-			double frequency = preferences.getDouble(String.format(frequencyPreferenceFormat, serial), 0);
+			double frequency = backend.getDouble(String.format(frequencyPreferenceFormat, serial), 0);
 			if (frequency == 0.0) {
-				int channel = preferences.getInt(String.format(channelPreferenceFormat, serial), 0);
+				int channel = backend.getInt(String.format(channelPreferenceFormat, serial), 0);
 				frequency = AltosConvert.radio_channel_to_frequency(channel);
 			}
 			frequencies.put(serial, frequency);
@@ -247,122 +248,122 @@ public class AltosPreferences {
 	}
 
 	public static void set_telemetry(int serial, int new_telemetry) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			telemetries.put(serial, new_telemetry);
-			preferences.putInt(String.format(telemetryPreferenceFormat, serial), new_telemetry);
+			backend.putInt(String.format(telemetryPreferenceFormat, serial), new_telemetry);
 			flush_preferences();
 		}
 	}
 
 	public static int telemetry(int serial) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			if (telemetries.containsKey(serial))
 				return telemetries.get(serial);
-			int telemetry = preferences.getInt(String.format(telemetryPreferenceFormat, serial),
-							   AltosLib.ao_telemetry_standard);
+			int telemetry = backend.getInt(String.format(telemetryPreferenceFormat, serial),
+						   AltosLib.ao_telemetry_standard);
 			telemetries.put(serial, telemetry);
 			return telemetry;
 		}
 	}
 
 	public static void set_scanning_telemetry(int new_scanning_telemetry) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			scanning_telemetry = new_scanning_telemetry;
-			preferences.putInt(scanningTelemetryPreference, scanning_telemetry);
+			backend.putInt(scanningTelemetryPreference, scanning_telemetry);
 			flush_preferences();
 		}
 	}
 
 	public static int scanning_telemetry() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return scanning_telemetry;
 		}
 	}
 
 	public static void set_voice(boolean new_voice) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			voice = new_voice;
-			preferences.putBoolean(voicePreference, voice);
+			backend.putBoolean(voicePreference, voice);
 			flush_preferences();
 		}
 	}
 
 	public static boolean voice() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return voice;
 		}
 	}
 
 	public static void set_callsign(String new_callsign) {
-		synchronized(preferences) {
+		synchronized(backend) {
 			callsign = new_callsign;
-			preferences.put(callsignPreference, callsign);
+			backend.putString(callsignPreference, callsign);
 			flush_preferences();
 		}
 	}
 
 	public static String callsign() {
-		synchronized(preferences) {
+		synchronized(backend) {
 			return callsign;
 		}
 	}
 
 	public static void set_firmwaredir(File new_firmwaredir) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			firmwaredir = new_firmwaredir;
-			preferences.put(firmwaredirPreference, firmwaredir.getPath());
+			backend.putString(firmwaredirPreference, firmwaredir.getPath());
 			flush_preferences();
 		}
 	}
 
 	public static File firmwaredir() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return firmwaredir;
 		}
 	}
 
 	public static void set_launcher_serial(int new_launcher_serial) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			launcher_serial = new_launcher_serial;
-			preferences.putInt(launcherSerialPreference, launcher_serial);
+			backend.putInt(launcherSerialPreference, launcher_serial);
 			flush_preferences();
 		}
 	}
 
 	public static int launcher_serial() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return launcher_serial;
 		}
 	}
 
 	public static void set_launcher_channel(int new_launcher_channel) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			launcher_channel = new_launcher_channel;
-			preferences.putInt(launcherChannelPreference, launcher_channel);
+			backend.putInt(launcherChannelPreference, launcher_channel);
 			flush_preferences();
 		}
 	}
 
 	public static int launcher_channel() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return launcher_channel;
 		}
 	}
 	
-	public static Preferences bt_devices() {
-		synchronized (preferences) {
-			return preferences.node("bt_devices");
+	public static AltosPreferencesBackend bt_devices() {
+		synchronized (backend) {
+			return backend.node("bt_devices");
 		}
 	}
 
 	public static AltosFrequency[] common_frequencies() {
-		synchronized (preferences) {
+		synchronized (backend) {
 			return common_frequencies;
 		}
 	}
 
 	public static void set_common_frequencies(AltosFrequency[] frequencies) {
-		synchronized(preferences) {
+		synchronized(backend) {
 			common_frequencies = frequencies;
 			save_common_frequencies(frequencies);
 			flush_preferences();
@@ -387,17 +388,38 @@ public class AltosPreferences {
 		set_common_frequencies(new_frequencies);
 	}
 
+	static LinkedList<AltosUnitsListener> units_listeners;
+
 	public static boolean imperial_units() {
-		synchronized(preferences) {
+		synchronized(backend) {
 			return AltosConvert.imperial_units;
 		}
 	}
 
 	public static void set_imperial_units(boolean imperial_units) {
-		synchronized (preferences) {
+		synchronized (backend) {
 			AltosConvert.imperial_units = imperial_units;
-			preferences.putBoolean(unitsPreference, imperial_units);
+			backend.putBoolean(unitsPreference, imperial_units);
 			flush_preferences();
 		}
+		if (units_listeners != null) {
+			for (AltosUnitsListener l : units_listeners) {
+				l.units_changed(imperial_units);
+			}
+		}
+	}
+
+	public static void register_units_listener(AltosUnitsListener l) {
+		synchronized(backend) {
+			if (units_listeners == null)
+				units_listeners = new LinkedList<AltosUnitsListener>();
+			units_listeners.add(l);
+		}
+	}
+
+	public static void unregister_units_listener(AltosUnitsListener l) {
+		synchronized(backend) {
+			units_listeners.remove(l);
+		}
 	}
 }
