@@ -15,14 +15,14 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.AltosLib;
+package org.altusmetrum.altoslib_1;
 
 public class AltosTelemetryRecordSatellite extends AltosTelemetryRecordRaw {
 	int		channels;
 	AltosGPSSat[]	sats;
 
-	public AltosTelemetryRecordSatellite(int[] in_bytes) {
-		super(in_bytes);
+	public AltosTelemetryRecordSatellite(int[] in_bytes, int rssi) {
+		super(in_bytes, rssi);
 
 		channels = uint8(5);
 		if (channels > 12)
