@@ -17,16 +17,11 @@
 
 package altosui;
 
-import java.awt.*;
-import java.awt.event.*;
 import javax.swing.*;
 import javax.swing.filechooser.FileNameExtensionFilter;
-import javax.swing.table.*;
 import java.io.*;
-import java.util.*;
-import java.text.*;
-import java.util.prefs.*;
-import org.altusmetrum.AltosLib.*;
+import org.altusmetrum.altoslib_1.*;
+import org.altusmetrum.altosuilib_1.*;
 
 public class AltosDataChooser extends JFileChooser {
 	JFrame	frame;
@@ -76,6 +71,12 @@ public class AltosDataChooser extends JFileChooser {
 	public AltosDataChooser(JFrame in_frame) {
 		frame = in_frame;
 		setDialogTitle("Select Flight Record File");
+		setFileFilter(new FileNameExtensionFilter("TeleMetrum eeprom file",
+							  "eeprom"));
+		setFileFilter(new FileNameExtensionFilter("Telemetry file",
+							  "telem"));
+		setFileFilter(new FileNameExtensionFilter("TeleMega eeprom file",
+							  "mega"));
 		setFileFilter(new FileNameExtensionFilter("Flight data file",
 							  "telem", "eeprom", "mega"));
 		setCurrentDirectory(AltosUIPreferences.logdir());
