@@ -29,6 +29,7 @@ __pdata uint32_t	ao_storage_config;
 /* Storage unit size - device reads and writes must be within blocks of this size. Usually 256 bytes. */
 __pdata uint16_t	ao_storage_unit;
 
+#define M25_DEBUG	0
 /*
  * Each flash chip is arranged in 64kB sectors; the
  * chip cannot erase in units smaller than that.
@@ -99,18 +100,7 @@ static __xdata uint8_t ao_m25_mutex;
 
 static __xdata uint8_t	ao_m25_instruction[4];
 
-#if HAS_BOOT_RADIO
-extern uint8_t ao_radio_in_recv;
-
-static void ao_boot_radio(void) {
-	if (ao_radio_in_recv)
-		ao_radio_recv_abort();
-}
-#else
-#define ao_boot_radio()
-#endif
-
-#define M25_SELECT(cs)		do { ao_boot_radio(); ao_spi_get_mask(AO_M25_SPI_CS_PORT,cs,AO_M25_SPI_BUS, AO_SPI_SPEED_FAST); } while (0)
+#define M25_SELECT(cs)		ao_spi_get_mask(AO_M25_SPI_CS_PORT,cs,AO_M25_SPI_BUS, AO_SPI_SPEED_FAST)
 #define M25_DESELECT(cs)	ao_spi_put_mask(AO_M25_SPI_CS_PORT,cs,AO_M25_SPI_BUS)
 
 #define M25_BLOCK_SHIFT			16
@@ -341,7 +331,9 @@ ao_storage_setup(void)
 void
 ao_storage_device_info(void) __reentrant
 {
+#if M25_DEBUG
 	uint8_t	cs;
+#endif
 #if M25_MAX_CHIPS > 1
 	uint8_t chip;
 #endif
@@ -359,6 +351,7 @@ ao_storage_device_info(void) __reentrant
 	printf ("Detected chips 1 size %d\n", ao_m25_total);
 #endif
 
+#if M25_DEBUG
 	printf ("Available chips:\n");
 	for (cs = 1; cs != 0; cs <<= 1) {
 		if ((AO_M25_SPI_CS_MASK & cs) == 0)
@@ -379,6 +372,7 @@ ao_storage_device_info(void) __reentrant
 			ao_m25_instruction[M25_UID_OFFSET]);
 		ao_mutex_put(&ao_m25_mutex);
 	}
+#endif
 }
 
 void
