@@ -38,8 +38,8 @@
 #define PACKET_HAS_MASTER	0
 #define PACKET_HAS_SLAVE	0
 
-#define AO_LED_CONTINUITY(c)	(1 << (c))
-#define AO_LED_CONTINUITY_MASK	(0xf)
+#define AO_LED_CONTINUITY(c)	(1 << ((c) + 2))
+#define AO_LED_CONTINUITY_MASK	(0xc)
 #define AO_LED_RX		0x10
 #define AO_LED_TX		0x20
 #define AO_LED_ARMED		0x40
@@ -74,40 +74,55 @@
 #define AO_PCA9922_CS_PIN	4
 #define AO_PCA9922_CS		P1_4
 
-#define AO_PAD_NUM		4
+#define AO_PAD_NUM		2
 #define	AO_PAD_PORT		P1
 #define AO_PAD_DIR		P1DIR
-#define AO_PAD_PIN_0		0
-#define AO_PAD_0		P1_0
-#define AO_PAD_PIN_1		1
-#define AO_PAD_1		P1_1
-#define AO_PAD_PIN_2		2
-#define AO_PAD_2		P1_2
-#define AO_PAD_PIN_3		3
-#define AO_PAD_3		P1_3
-#define AO_PAD_ALL_PINS		((1 << AO_PAD_PIN_0) | (1 << AO_PAD_PIN_1) | (1 << AO_PAD_PIN_2) | (1 << AO_PAD_PIN_3))
+
+#define AO_PAD_PIN_0		2
+#define AO_PAD_0		P1_2
+#define AO_PAD_ADC_0		2
+
+#define AO_PAD_PIN_1		3
+#define AO_PAD_1		P1_3
+#define AO_PAD_ADC_1		3
+
+#define AO_PAD_ALL_PINS		((1 << AO_PAD_PIN_0) | (1 << AO_PAD_PIN_1))
+#define AO_PAD_ALL_CHANNELS	((1 << 0) | (1 << 1))
+
+#define AO_SIREN_PORT		P1
+#define AO_SIREN_DIR		P1DIR
+#define AO_SIREN_PIN		0
+#define AO_SIREN		P1_0
+
+#define AO_STROBE_PORT		P1
+#define AO_STROBE_DIR		P1DIR
+#define AO_STROBE_PIN		1
+#define AO_STROBE		P1_1
 
 /* test these values with real igniters */
 #define AO_PAD_RELAY_CLOSED	3524
 #define AO_PAD_NO_IGNITER	16904
 #define AO_PAD_GOOD_IGNITER	22514
 
+#define AO_PAD_ADC_PYRO		4
+#define AO_PAD_ADC_BATT		5
+
+#define AO_ADC_FIRST_PIN	2
+
 struct ao_adc {
-	int16_t		sense[4];
+	int16_t		sense[AO_PAD_NUM];
 	int16_t		pyro;
 	int16_t		batt;
 };
 
 #define AO_ADC_DUMP(p)							\
-	printf ("tick: %5u 0: %5d 1: %5d 2: %5d 3: %5d pyro: %5d batt %5d\n", \
+	printf ("tick: %5u 0: %5d 1: %5d pyro: %5d batt %5d\n", 	\
 		(p)->tick,						\
 		(p)->adc.sense[0],					\
 		(p)->adc.sense[1],					\
-		(p)->adc.sense[2],					\
-		(p)->adc.sense[3],					\
 		(p)->adc.pyro,						\
 		(p)->adc.batt)
 
-#define AO_ADC_PINS	((1 << 0) | (1 << 1) | (1 << 2) | (1 << 3) | (1 << 4) | (1 << 5))
+#define AO_ADC_PINS	((1 << AO_PAD_ADC_0) | (1 << AO_PAD_ADC_1) | (1 << AO_PAD_ADC_PYRO) | (1 << AO_PAD_ADC_BATT))
 
 #endif /* _AO_PINS_H_ */
