@@ -49,18 +49,19 @@
 #define HAS_TELEMETRY		0
 #define HAS_AES			1
 
-#define HAS_SPI_1		1
-#define SPI_1_PA5_PA6_PA7	1
+#define HAS_SPI_1		0
+#define SPI_1_PA5_PA6_PA7	0
 #define SPI_1_PB3_PB4_PB5	0
 #define SPI_1_PE13_PE14_PE15	0
 
-#define HAS_SPI_2		1
-#define SPI_2_PB13_PB14_PB15	1
-#define SPI_2_PD1_PD3_PD4	0
-#define SPI_2_GPIO		(&stm_gpiob)
-#define SPI_2_SCK		13
-#define SPI_2_MISO		14
-#define SPI_2_MOSI		15
+#define HAS_SPI_2		1	/* CC1120 */
+#define SPI_2_PB13_PB14_PB15	0
+#define SPI_2_PD1_PD3_PD4	1
+#define SPI_2_GPIO		(&stm_gpiod)
+#define SPI_2_SCK		1
+#define SPI_2_MISO		3
+#define SPI_2_MOSI		4
+#define SPI_2_OSPEEDR		STM_OSPEEDR_10MHz
 
 #define HAS_I2C_1		0
 
@@ -70,29 +71,43 @@
 #define PACKET_HAS_MASTER	0
 
 /*
- * Radio is a cc1111 connected via SPI
+ * Radio is a cc1120 connected via SPI
  */
-#define AO_RADIO_CAL_DEFAULT	1186611
 
-#define AO_RADIO_SPI_BUS	AO_SPI_2_PB13_PB14_PB15
-#define AO_RADIO_CS_PORT	(&stm_gpiob)
-#define AO_RADIO_CS_PIN		12
+#define AO_RADIO_CAL_DEFAULT 	0x6ca333
+
+#define AO_FEC_DEBUG		0
+#define AO_CC1120_SPI_CS_PORT	(&stm_gpiod)
+#define AO_CC1120_SPI_CS_PIN	0
+#define AO_CC1120_SPI_BUS	AO_SPI_2_PD1_PD3_PD4
+#define AO_CC1120_SPI		stm_spi2
+
+#define AO_CC1120_INT_PORT		(&stm_gpioc)
+#define AO_CC1120_INT_PIN		(15)
+#define AO_CC1120_MCU_WAKEUP_PORT	(&stm_gpiod)
+#define AO_CC1120_MCU_WAKEUP_PIN	(2)
+
+#define AO_CC1120_INT_GPIO	2
+#define AO_CC1120_INT_GPIO_IOCFG	CC1120_IOCFG2
+
+#define AO_CC1120_MARC_GPIO	3
+#define AO_CC1120_MARC_GPIO_IOCFG	CC1120_IOCFG3
 
-#define AO_RADIO_INT_PORT	(&stm_gpioc)
-#define AO_RADIO_INT_PIN	14
 
 #define LOW_LEVEL_DEBUG		0
 
 #define LED_PORT_ENABLE		STM_RCC_AHBENR_GPIOCEN
 #define LED_PORT		(&stm_gpioc)
 #define LED_PIN_RED		7
-#define LED_PIN_GREEN		8
-#define LED_PIN_CONTINUITY_3	9
-#define LED_PIN_CONTINUITY_2	10
-#define LED_PIN_CONTINUITY_1	11
-#define LED_PIN_CONTINUITY_0	12
-#define LED_PIN_REMOTE_ARM	13
+#define LED_PIN_AMBER		8
+#define LED_PIN_GREEN		9
+#define LED_PIN_CONTINUITY_3	10
+#define LED_PIN_CONTINUITY_2	11
+#define LED_PIN_CONTINUITY_1	12
+#define LED_PIN_CONTINUITY_0	13
+#define LED_PIN_REMOTE_ARM	14
 #define AO_LED_RED		(1 << LED_PIN_RED)
+#define AO_LED_AMBER		(1 << LED_PIN_AMBER)
 #define AO_LED_GREEN		(1 << LED_PIN_GREEN)
 #define AO_LED_CONTINUITY_3	(1 << LED_PIN_CONTINUITY_3)
 #define AO_LED_CONTINUITY_2	(1 << LED_PIN_CONTINUITY_2)
@@ -104,6 +119,7 @@
 #define AO_LED_REMOTE_ARM	(1 << LED_PIN_REMOTE_ARM)
 
 #define LEDS_AVAILABLE		(AO_LED_RED |		\
+				 AO_LED_AMBER |		\
 				 AO_LED_GREEN |		\
 				 AO_LED_CONTINUITY_3 |	\
 				 AO_LED_CONTINUITY_2 |	\
@@ -113,6 +129,9 @@
 
 /* LCD displays */
 
+#define LCD_DEBUG		0
+#define SEVEN_SEGMENT_DEBUG	0
+
 #define AO_LCD_STM_SEG_ENABLED_0 (		\
 		(1 << 0) | /* PA1 */		\
 		(1 << 1) | /* PA2 */		\
@@ -122,22 +141,22 @@
 		(1 << 5) | /* PB0 */		\
 		(1 << 6) | /* PB1 */		\
 		(1 << 7) | /* PB3 */		\
-		(0 << 8) | /* PB4 */		\
-		(0 << 9) | /* PB5 */		\
-		(0 << 10) | /* PB10 */		\
-		(0 << 11) | /* PB11 */		\
-		(0 << 12) | /* PB12 */		\
-		(0 << 13) | /* PB13 */		\
-		(0 << 14) | /* PB14 */		\
-		(0 << 15) | /* PB15 */		\
-		(0 << 16) | /* PB8 */		\
-		(0 << 17) | /* PA15 */		\
-		(0 << 18) | /* PC0 */		\
-		(0 << 19) | /* PC1 */		\
-		(0 << 20) | /* PC2 */		\
-		(0 << 21) | /* PC3 */		\
-		(0 << 22) | /* PC4 */		\
-		(0 << 23) | /* PC5 */		\
+		(1 << 8) | /* PB4 */		\
+		(1 << 9) | /* PB5 */		\
+		(1 << 10) | /* PB10 */		\
+		(1 << 11) | /* PB11 */		\
+		(1 << 12) | /* PB12 */		\
+		(1 << 13) | /* PB13 */		\
+		(1 << 14) | /* PB14 */		\
+		(1 << 15) | /* PB15 */		\
+		(1 << 16) | /* PB8 */		\
+		(1 << 17) | /* PA15 */		\
+		(1 << 18) | /* PC0 */		\
+		(1 << 19) | /* PC1 */		\
+		(1 << 20) | /* PC2 */		\
+		(1 << 21) | /* PC3 */		\
+		(1 << 22) | /* PC4 */		\
+		(1 << 23) | /* PC5 */		\
 		(0 << 24) | /* PC6 */		\
 		(0 << 25) | /* PC7 */		\
 		(0 << 26) | /* PC8 */		\
@@ -159,8 +178,8 @@
 
 #define AO_LCD_STM_COM_ENABLED (		\
 		(1 << 0) | /* PA8 */		\
-		(1 << 1) | /* PA9 */		\
-		(1 << 2) | /* PA10 */		\
+		(0 << 1) | /* PA9 */		\
+		(0 << 2) | /* PA10 */		\
 		(0 << 3) | /* PB9 */		\
 		(0 << 4) | /* PC10 */		\
 		(0 << 5) | /* PC11 */		\
@@ -168,16 +187,42 @@
 
 #define AO_LCD_28_ON_C	0
 
-#define AO_LCD_DUTY	STM_LCD_CR_DUTY_1_4
+#define AO_LCD_DUTY	STM_LCD_CR_DUTY_STATIC
 
-#define AO_SEGMENT_0		0
-#define AO_SEGMENT_1		5
-#define AO_SEGMENT_2		1
-#define AO_SEGMENT_3		6
-#define AO_SEGMENT_4		4
-#define AO_SEGMENT_5		2
-#define AO_SEGMENT_6		3
-#define AO_SEGMENT_7		7
+#define AO_LCD_PER_DIGIT	1
+
+#define AO_LCD_DIGITS		3
+#define AO_LCD_SEGMENTS		8
+
+#define AO_SEGMENT_MAP {			\
+		/* pad segments */		\
+		{ 0, 14 },			\
+		{ 0, 13 },			\
+		{ 0, 15 },			\
+		{ 0, 17 },			\
+		{ 0, 16 },			\
+		{ 0, 8 },			\
+		{ 0, 9 },			\
+		{ 0, 7 },			\
+		/* box1 segments */		\
+		{ 0, 10 },			\
+		{ 0, 6 },			\
+		{ 0, 11 },			\
+		{ 0, 12 },			\
+		{ 0, 21 },			\
+		{ 0, 19 },			\
+		{ 0, 20 },			\
+		{ 0, 18 },			\
+		/* box0 segments */		\
+		{ 0, 22 },			\
+		{ 0, 4 },			\
+		{ 0, 23 },			\
+		{ 0, 5 },			\
+		{ 0, 3 },			\
+		{ 0, 1 },			\
+		{ 0, 2 },			\
+		{ 0, 0 },			\
+}
 
 /*
  * Use event queue for input devices
@@ -192,13 +237,13 @@
 #define AO_QUADRATURE_COUNT	2
 #define AO_QUADRATURE_MODE	0
 
-#define AO_QUADRATURE_0_PORT	&stm_gpioc
+#define AO_QUADRATURE_0_PORT	&stm_gpioe
 #define AO_QUADRATURE_0_A	3
 #define AO_QUADRATURE_0_B	2
 
 #define AO_QUADRATURE_PAD	0
 
-#define AO_QUADRATURE_1_PORT	&stm_gpioc
+#define AO_QUADRATURE_1_PORT	&stm_gpioe
 #define AO_QUADRATURE_1_A	1
 #define AO_QUADRATURE_1_B	0
 
@@ -211,12 +256,12 @@
 #define AO_BUTTON_COUNT		2
 #define AO_BUTTON_MODE		AO_EXTI_MODE_PULL_UP
 
-#define AO_BUTTON_0_PORT	&stm_gpioc
+#define AO_BUTTON_0_PORT	&stm_gpioe
 #define AO_BUTTON_0		4
 
 #define AO_BUTTON_ARM		0
 
-#define AO_BUTTON_1_PORT	&stm_gpioc
+#define AO_BUTTON_1_PORT	&stm_gpioe
 #define AO_BUTTON_1		5
 
 #define AO_BUTTON_FIRE		1
